import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { request } from 'http';
import { DirectPost } from 'src/helper/nmi-payment-gatway-helper';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { PaymentConfigurationRepository } from '../payment-configuration/payment-configuration.repository';
import { CreatePaymentGatewayDto } from './dto/create-payment-gateway.dto';
import { UpdatePaymentGatewayDto } from './dto/update-payment-gateway.dto';
import { PaymentGatewayRepository } from './payment-gateway.repository';

@Injectable()
export class PaymentGatewayService {
    constructor(
        @InjectRepository(PaymentConfigurationRepository)
        private readonly paymentConfigurationRepository: PaymentConfigurationRepository,
        @InjectRepository(PaymentGatewayRepository)
        private readonly paymentGatewayRepository: PaymentGatewayRepository,
    ) { }

    async payment(dto: CreatePaymentGatewayDto, request: any): Promise<any | Error> {
        const paymentConf = await this.paymentConfigurationRepository.find();
        if (!paymentConf) {
            throw new HttpException(
                "Payment security key not found",
                HttpStatus.NOT_FOUND
            );
        }
        const dp = new DirectPost(paymentConf[0].securityKey);

        const billingInfo = {
            'first_name': dto.billingInfo.firstName,
            'last_name': dto.billingInfo.lastName,
            'address1': dto.billingInfo.addressOne,
            'city': dto.billingInfo.city,
            'state': dto.billingInfo.state,
            'zip': dto.billingInfo.zip
        }
        const shippingInfo = {
            'shipping_first_name': dto.shippingInfo.firstName,
            'shipping_last_name': dto.shippingInfo.lastName,
            'shipping_address1': dto.shippingInfo.addressOne,
            'shipping_city': dto.shippingInfo.city,
            'shipping_state': dto.shippingInfo.state,
            'shipping_zip': dto.shippingInfo.zip
        }
        dp.setBilling(billingInfo);
        dp.setShipping(shippingInfo);
        const data = await dp.doSale(dto.amount, dto.ccNum, dto.ccExp, dto.cvv);
        if (data['statusCode'] === 200) {
            const payload = {
                customerId: request.user.id,
                customerName: request.user.firstName + ' ' + request.user.lastName,
                invoiceID: dto.invoiceID,
                paymentAmount: dto.amount,
                paymentDate: new Date(),
                paymentTransID: data['transactionid'],
                paymentPayload: JSON.stringify(data),
                transStatus: "Success",
            }
            const res = await this.paymentGatewayRepository.createEntity(payload);
            return ResponseUtils.successResponseHandler(200, "Payment done successfully", "data", res);
        }
        else return ResponseUtils.errorResponseHandler(400, "Error occured!", "data", data);
    }

    async refund(id: string, dto: any): Promise<any | Error> {
        const paymentConf = await this.paymentConfigurationRepository.find();
        if (!paymentConf) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const dp = new DirectPost(paymentConf[0].securityKey);
        const data = await dp.doRefund(dto.amount, dto.transactionID);
        if (data['statusCode'] === 200) {
            const payload = {
                refundTransID: data['transactionid'],
                refundPayload: JSON.stringify(data),
                isRefunded: true,
                transStatus: true,
            }
            const res = await this.paymentGatewayRepository.updateEntity(id, payload);
            return ResponseUtils.successResponseHandler(200, "Refund done successfully", "data", res);
        }
        else return ResponseUtils.errorResponseHandler(400, "Error occured!", "data", data);
    }

    async findAll(
        transId: string,
        startDate: Date,
        endDate: Date,
        userName: string,
        orderId: string
    ): Promise<any[] | Error> {
        let query: any = {};
        if (transId) query.paymentTransID = transId;
        if (startDate) query.startDate = startDate;
        if (endDate) query.endDate = endDate;
        if (userName) query.customerName = userName;
        if (orderId) query.invoiceID = orderId;
        const data = await this.paymentGatewayRepository.findAllEntities(query);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Payment Log Data Fetched", "data", data);
    }

    // async professionalPaymentLog(
    //     request: any
    // ): Promise<any[] | Error> {
    //     let response = [];
    //     const invoice = await this.invoiceRepository.findByFilterQuery({ professionalId: request.user.id });
    //     if (!invoice) {
    //         throw new HttpException(
    //             Constants.NOT_FOUND,
    //             HttpStatus.NOT_FOUND
    //         );
    //     }
    //     for (let i = 0; i < Object.keys(invoice).length; i++) {
    //         const data = await this.paymentGatewayRepository.find({
    //             order: {createdAt: 'DESC'},
    //             where: {
    //                 invoiceID: invoice[i].invoiceId,
    //                 isDeleted: false
    //             }
    //         });
    //         if (data) {
    //             if (Object.keys(data).length > 1) {
    //                 for (let j = 0; j < Object.keys(data).length; j++) {
    //                     response.push(data[j]);
    //                 }
    //             } else {
    //                 response.push(data[0]);
    //             }
    //         }
    //     }
    //     return ResponseUtils.successResponseHandler(200, "Payment Log Data Fetched", "data", response);
    // }
}
